<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sch="http://www.ascc.net/xml/schematron"  xmlns:xsd="http://www.w3.org/2001/XMLSchema">

	<!-- Set the output to be XML with an XML declaration and use indentation -->
	<xsl:output method="xml" omit-xml-declaration="no" indent="yes" standalone="yes"/>
	
	<!-- Remove all white-space only nodes from the XSD document -->
	<xsl:strip-space elements="*"/>

	<!-- Match the xsd:schema element, output the sch:schema element and extract the schematron information -->
	<xsl:template match="xsd:schema">
		<sch:schema>
		    <!-- Extract all information from all xsd:appinfo elements in the XSD document. -->
		    <xsl:apply-templates select="//xsd:appinfo/*[starts-with(name(.), 'sch:') and not(name(.) = 'sch:diagnostics')]" mode="not-diagnostics"/>
		    <!-- Diagnostics always go last. -->
		    <xsl:apply-templates select="//xsd:appinfo/sch:diagnostics" mode="diagnostics"/>
		</sch:schema>
	</xsl:template>
	
	<xsl:template match="xsd:appinfo">
		<xsl:apply-templates/>
	</xsl:template>

        <xsl:template match="*[starts-with(name(.), 'sch:')]" mode="not-diagnostics">
            <xsl:element name="{name(.)}">
                <xsl:for-each select="@*">
                    <xsl:attribute name="{name(.)}">
                        <xsl:value-of select="."/>
                    </xsl:attribute>
                </xsl:for-each>
                <xsl:apply-templates mode="not-diagnostics"/>
            </xsl:element>
        </xsl:template>
	
        <xsl:template match="*[not(starts-with(name(.), 'sch:'))]" mode="not-diagnostics"/>

        <xsl:template match="*" mode="diagnostics">
            <xsl:element name="{name(.)}">
                <xsl:for-each select="@*">
                    <xsl:attribute name="{name(.)}">
                        <xsl:value-of select="."/>
                    </xsl:attribute>
                </xsl:for-each>
                <xsl:apply-templates mode="diagnostics"/>
            </xsl:element>
        </xsl:template>

	<xsl:template match="comment()"/>

</xsl:stylesheet>

