<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:sch="http://www.ascc.net/xml/schematron"  xmlns:xsd="http://www.w3.org/2000/10/XMLSchema">

	<!-- Set the output to be XML with an XML declaration and use indentation -->
	<xsl:output method="xml" omit-xml-declaration="no" indent="yes" standalone="yes"/>
	
	<!-- Remove all white-space only nodes from the XSD document -->
	<xsl:strip-space elements="*"/>

	<!-- Match the xsd:schema element, output the sch:schema element and extract the schematron information -->
	<xsl:template match="xsd:schema">
		<sch:schema>
			<!-- Extract all information from all xsd:appinfo elements in the XSD document. -->
			<xsl:call-template name="extract-info">
				<xsl:with-param name="appinfo" select="//xsd:appinfo"/>
			</xsl:call-template>
		</sch:schema>
	</xsl:template>
	
	<!-- Template that calls the recursive function for retrieving the xsd:appinfo element. Pass all the xsd:appinfo nodes to the template. -->
	<xsl:template name="extract-info">
		<xsl:param name="appinfo"/>
		<xsl:call-template name="extract-element">
			<xsl:with-param name="elementContent" select="$appinfo"/>
		</xsl:call-template>
	</xsl:template>

	<!-- Recursive function that extracts the elements (with the prefix sch:) and text nodes from the xsd:appinfo element and writes the result to the output file. Pass all the xsd:appinfo nodes to the template. -->
	<xsl:template name="extract-element">
		<xsl:param name="elementContent"/>
		<!-- Iterate through all the xsd:appinfo element nodes and if an element with the prefix sch: is found an element is created together with its attributes. The element content is then extracted with a recursive call to this function with the content of the current element as parameter. -->
		<xsl:for-each select="($elementContent/node())">
			<xsl:if test="contains(name(), 'sch:')">
				<xsl:element name="{name()}">
					<xsl:for-each select="@*">
						<xsl:attribute name="{name()}">
							<xsl:value-of select="."/>
						</xsl:attribute>
					</xsl:for-each>
					<xsl:call-template name="extract-element">
						<xsl:with-param name="elementContent" select="."/>
					</xsl:call-template>
				</xsl:element>
			</xsl:if>
			<xsl:if test="not(contains(name(), 'sch:'))">
                           <xsl:value-of select="."/>
                       </xsl:if>
		</xsl:for-each>
		<!-- Iterate through all the text nodes in the appinfo element and write the value to the output file. -->
		<!--xsl:for-each select="$elementContent/text()">
			<xsl:value-of select="."/>
		</xsl:for-each-->
	</xsl:template>
	
</xsl:stylesheet>

